#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "fuggveny.h"

/** A koordintarendszer rajzolshoz hasznlt szn. */
static const char alapszin[] = "black";
/** A nyl s az egysgjel mrete a koordintarendszeren */
static const int nyilmeret = 5;

/** Kperny koordintbl matematikai koordintt szmol.
 * @param r A rajz, amin dolgozik. Ebbl veszi ki a nagytst s az orig adatait.
 * @param kx A kperny x koordinta, amit t kell szmolni.
 * @param ky A kperny y koordinta, amit t kell szmolni.
 * @param mx Pointer a matematikai x koordintra, ahova az eredmny kerl, vagy NULL, ha nincs r szksg.
 * @param my Pointer a matematikai y koordintra, ahova az eredmny kerl, vagy NULL, ha nincs r szksg.
 */
static void kep2mat(Rajz const *r, double kx, double ky, double *mx, double *my)
{
    if (mx!=NULL)
        *mx = (kx - r->ox) / r->nx;
    if (my!=NULL)
        *my = -(ky - r->oy) / r->ny;
}


/** Matematikai koordintbl kpkoordintt szmt.
 * Ha a kx, ky kimeneti paramterek valamelyikben NULL pointert kap, akkor
 * azt a koordintt nem trolja el.
 * @param r A rajz, amin dolgozik. Ebbl veszi ki a nagytst s az orig adatait.
 * @param mx A matematikai x koordinta, amit t kell szmolni.
 * @param my A matematikai y koordinta, amit t kell szmolni.
 * @param kx Pointer a kperny x koordintra, ahova az eredmny kerl, vagy NULL.
 * @param ky Pointer a kperny y koordintra, ahova az eredmny kerl, vagy NULL.
 */
static void mat2kep(Rajz const *r, double mx, double my, double *kx, double *ky)
{
    if (kx!=NULL)
        *kx = r->ox + mx*r->nx;
    if (ky!=NULL)
        *ky = r->oy - my*r->ny;
}


/**
 * Szakaszt rajzol egy rajzra.
 * @param x1 A szakasz egyik vgpontjnak x koordintja.
 * @param y1 A szakasz egyik vgpontjnak y koordintja.
 * @param x1 A szakasz msik vgpontjnak x koordintja.
 * @param y1 A szakasz msik vgpontjnak y koordintja.
 * @param szin A szakasz szne, egy SDL-kompatibilis megadssal (pl. "red" vagy "#009900").
 * @return 0, ha nincs tbb hely a rajzon, 1, ha szakasz hozzadva.
 */
static int szakasz_rajzol(Rajz *r, double x1, double y1, double x2, double y2, char const *szin)
{
    Szakasz uj;

    /* Megtelt a tmb? Hiba! */
    if (r->db == SZAKASZ_MAX)
        return 0;
    uj.x1 = x1;
    uj.y1 = y1;
    uj.x2 = x2;
    uj.y2 = y2;
    strcpy(uj.szin, szin);
    /* Betesszk a tmbbe s nveljk a darabszmot */
    r->szakaszok[r->db++] = uj;

    /* Minden ok */
    return 1;
}


/* j rajzot kezd. */
void rajz_inicializal(Rajz *r, int kx, int ky, double ox, double oy, double nx, double ny)
{
    /* Megjegyzi a nagytst s az eltolst */
    r->kx = kx;
    r->ky = ky;
    r->ox = ox;
    r->oy = oy;
    r->nx = nx;
    r->ny = ny;
    /* s most ugyanaz, mintha trlnnk */
    rajz_torol(r);
}


/* Mr inicializlt rajzot trl. */
void rajz_torol(Rajz *r)
{
    double ex, ey;

    /* Kezdetben nincs rajta vonal */
    r->db = 0;
    /* Koordintarendszer, x tengely */
    szakasz_rajzol(r, 0, r->oy, r->kx, r->oy, alapszin);
    szakasz_rajzol(r, r->kx, r->oy, r->kx-nyilmeret*2, r->oy-nyilmeret, alapszin);
    szakasz_rajzol(r, r->kx, r->oy, r->kx-nyilmeret*2, r->oy+nyilmeret, alapszin);
    /* Koordintarendszer, y tengely */
    szakasz_rajzol(r, r->ox, 0, r->ox, r->ky, alapszin);
    szakasz_rajzol(r, r->ox, 0, r->ox-nyilmeret, nyilmeret*2, alapszin);
    szakasz_rajzol(r, r->ox, 0, r->ox+nyilmeret, nyilmeret*2, alapszin);
    /* egysgek */
    mat2kep(r, 1, 1, &ex, &ey);
    szakasz_rajzol(r, ex, r->oy-nyilmeret, ex, r->oy+nyilmeret, alapszin);
    szakasz_rajzol(r, r->ox-nyilmeret, ey, r->ox+nyilmeret, ey, alapszin);
}


/* Kirajzol egy fggvnyt. */
int fuggveny_rajzol(Rajz *r, double (*fv)(double), char const *szin)
{
    double const lepes = 1.0;
    double xk, yk, xelozo, yelozo;  /* kp koordintk */

    /* Haladjunk a kp koordinti szerint, gy pont olyan
     * rszletessget kapunk, mint amekkora a kp. */
    for (xk=0; xk<=r->kx; xk+=lepes) {
        double xmat, ymat;

        /* Melyik matematikai x koordinta felel meg ennek? */
        kep2mat(r, xk, 0, &xmat, NULL);
        /* Azt kell a fggvnybe behelyettesteni */
        ymat=fv(xmat);
        /* Visszaszmolni kpkoordintkra megint */
        mat2kep(r, xmat, ymat, NULL, &yk);
        /* Utna rajz (kivtel a legels helyen, mert azeltt
         * mg nincs elz pont). */
        if (xk!=0)
            if (!szakasz_rajzol(r, xk, yk, xelozo, yelozo, szin))
                /* ha megtelt a trol, akkor visszatrnk hamis rtkkel */
                return 0;

        /* a kvetkez iterci szmra */
        xelozo = xk;
        yelozo = yk;
    }

    /* sikerlt teljesen kirajzolni, visszatrnk igaz rtkkel */
    return 1;
}


/* Nyitott fjlba rakja az SVG elemeket. */
void rajz_kiir(Rajz const *r, FILE *fp)
{
    int i;

    fprintf(fp, "<svg width=\"%d\" height=\"%d\" xmlns=\"http://www.w3.org/2000/svg\">\n",
        r->kx, r->ky);
    for (i=0; i<r->db; ++i)
        fprintf(fp, "  <line x1=\"%g\" y1=\"%g\" x2=\"%g\" y2=\"%g\" stroke=\"%s\" />\n",
            r->szakaszok[i].x1, r->szakaszok[i].y1,
            r->szakaszok[i].x2, r->szakaszok[i].y2,
            r->szakaszok[i].szin);
    fprintf(fp, "</svg>\n");
}


/* Adott nev fjlba rja a rajzot. */
void rajz_fajlba(Rajz const *r, char const *nev)
{
    FILE *fp;

    fp = fopen(nev, "wt");
    if (fp==NULL)   /* Nem sikerlt megnyitni? */
        return;
    rajz_kiir(r, fp);
    fclose(fp);
}
