#ifndef FUGGVENY_H
#define FUGGVENY_H

#include <stdio.h>

/** A Szakasz struktra egy szakasz adatait trolja egy rajzon. */
typedef struct Szakasz {
    /** Koordintk, mr a kp koordintarendszerben. */
    double x1, y1, x2, y2;
    /** A szakasz szne. */
    char szin[20+1];
} Szakasz;


/** Maximum ennyi szakaszbl llhat egy rajz. Azrt az enumos trkk,
 * mert gy lehet tmb mretnek is hasznlni. */
enum { SZAKASZ_MAX = 2000 };


/** A Rajz struktra egy (vagy tbb) megrajzolt fggvny
 * adatait tartalmazza, lnyegben magt a kpet. */
typedef struct Rajz {
    /** Kp mrete kppontokban */
    int kx, ky;
    /** Orig - mely kpkoordintkon van az orig */
    double ox, oy;
    /** Nagyts - hny karakter jelenti az egysget */
    double nx, ny;
    /** A kp szakaszainak szma. */
    int db;
    /** A kp szakaszai. Csak az els 0..db-1-ben van adat. */
    Szakasz szakaszok[SZAKASZ_MAX];
} Rajz;


/** j rajzot hoz ltre.
 * Inicializlhat vele memriaszemetet tartalmaz rajz struktra is,
 * de egy mr hasznlt rajz struktra trlsre is hasznlhat.
 * @param r Pointer a rajz struktrra, amit inicializlni kell.
 * @param ox Az orig x koordintja, kp koordintarendszerben.
 * @param oy Az orig y koordintja, kp koordintarendszerben.
 * @param nx x irny nagyts: hny kpernyn lthat kppont az egysg vzszintesen.
 * @param ny y irny nagyts: hny kpernyn lthat kppont az egysg fgglegesen.
 */
void rajz_inicializal(Rajz *r, int kx, int ky, double ox, double oy, double nx, double ny);


/**
 * j rajzot kezd, egy res koordintarendszerrel.
 * A rajzon lv eddigi sszes fggvny trldik, s egy res koordintarendszert
 * rajzol bele a fggvny.
 * @param r A trlend rajz. Nem lehet memriaszemt, mr inicializlva kell legyen.
 */
void rajz_torol(Rajz *r);


/** Kirja a paramterknt kapott fjlba egy rajz tartalmt SVG-ben.
 * @param r A rajz, amit meg kell jelenteni.
 * @param fp Az rsra megnyitott szvegfjl, ahova rni kell a rajzot. */
void rajz_kiir(Rajz const *r, FILE *fp);


/** Kirja egy SVG fjlba a rajz tartalmt.
 * @param r A rajz, amit meg kell jelenteni.
 * @param fp A szvegfjl neve. */
void rajz_fajlba(Rajz const *r, char const *nev);


/**
 * brzolja a paramterknt kapott fggvnyt.
 * @param r Rajz, amire a fggvnyt brzolni kell. Ebbl veszi a nagyts adatait is.
 * @param fv Pointer a fggvnyre, amit brzolni kell (double paramter, double visszatrsi rtk).
 * @param szin A szn, amivel rajzolni kell (pl. "red").
 * @return HAMIS, ha rajzols kzben megtelt a rajz trolja, IGAZ, ha minden rendben volt
 */
int fuggveny_rajzol(Rajz *r, double (*fv)(double), char const *szin);

#endif
